@@ -913,7 +913,8 @@ strdup_len (const gchar *string,
 
 /**
  * g_locale_to_utf8:
- * @opsysstring:   a string in the encoding of the current locale
+ * @opsysstring:   a string in the encoding of the current locale. On Windows
+ *                 this means the system codepage.
  * @len:           the length of the string, or -1 if the string is
  *                 nul-terminated.
  * @bytes_read:    location to store the number of bytes in the
@@ -1069,7 +1070,7 @@ g_get_filename_charsets (G_CONST_RETURN gchar ***filename_charsets)
       cache->charset = g_strdup (charset);
       
       p = getenv ("G_FILENAME_ENCODING");
-      if (p != NULL) 
+      if (p != NULL && p[0] != '\0') 
 	{
 	  cache->filename_charsets = g_strsplit (p, ",", 0);
 	  cache->is_utf8 = (strcmp (cache->filename_charsets[0], "UTF-8") == 0);
@@ -1722,8 +1723,9 @@ g_filename_from_uri (const gchar *uri,
 
 /**
  * g_filename_to_uri:
- * @filename: an absolute filename specified in the encoding
- *            used for filenames by the operating system.
+ * @filename: an absolute filename specified in the GLib file name encoding,
+ *            which is the on-disk file name bytes on Unix, and UTF-8 on 
+ *            Windows
  * @hostname: A UTF-8 encoded hostname, or %NULL for none.
  * @error: location to store the error occuring, or %NULL to ignore
  *         errors. Any of the errors in #GConvertError may occur.
