@@ -52,6 +52,7 @@ typedef enum
   G_FILE_ERROR_INTR,
   G_FILE_ERROR_IO,
   G_FILE_ERROR_PERM,
+  G_FILE_ERROR_NOSYS,
   G_FILE_ERROR_FAILED
 } GFileError;
 
@@ -72,6 +73,13 @@ GQuark     g_file_error_quark      (void);
 /* So other code can generate a GFileError */
 GFileError g_file_error_from_errno (gint err_no);
 
+#ifdef G_OS_WIN32
+#define g_file_test g_file_test_utf8
+#define g_file_get_contents g_file_get_contents_utf8
+#define g_mkstemp g_mkstemp_utf8
+#define g_file_open_tmp g_file_open_tmp_utf8
+#endif
+
 gboolean g_file_test         (const gchar  *filename,
                               GFileTest     test);
 gboolean g_file_get_contents (const gchar  *filename,
@@ -98,5 +106,3 @@ gchar *g_build_filename (const gchar *first_element,
 G_END_DECLS
 
 #endif /* __G_FILEUTILS_H__ */
-
-
