@@ -37,6 +37,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <wchar.h>
 #include <errno.h>
 
 #define STRICT			/* Strict typing, please */
@@ -1064,7 +1065,7 @@ g_win32_error_message (gint error)
 		      |FORMAT_MESSAGE_IGNORE_INSERTS
 		      |FORMAT_MESSAGE_FROM_SYSTEM,
 		      NULL, error, 0,
-		      (LPTSTR) &msg, 0, NULL);
+		      (LPWSTR) &msg, 0, NULL);
       if (msg != NULL)
 	{
 	  nchars = wcslen (msg);
